import logging
from typing import Text, Dict, Union, List, Optional

from sqlalchemy.orm import Session

from rasax.community import config
from rasax.community.database import PlatformFeature

logger = logging.getLogger(__name__)


class FeatureService(object):
    def __init__(self, session: Session):
        self.session = session

    def set_feature(self, feature: Dict[Text, Union[Text, bool]]) -> None:
        platform_feature = (
            self.session.query(PlatformFeature)
            .filter(PlatformFeature.feature_name == feature["name"])
            .first()
        )

        if platform_feature:
            platform_feature.enabled = feature["enabled"]
        else:
            platform_feature = PlatformFeature(
                feature_name=feature["name"], enabled=feature["enabled"]
            )
            self.session.add(platform_feature)

        self.session.commit()

    @staticmethod
    def _is_contained_in_list(
        feature_name: Text, feature_list: List[Dict[Text, Union[Text, bool]]]
    ) -> Optional[Dict[Text, Union[Text, bool]]]:
        for r in feature_list:
            if r.get("name") == feature_name:
                return r
        return None

    def _merge_with_defaults(
        self, existing_feature_list: List[Dict[Text, Union[Text, bool]]]
    ) -> List[Dict[Text, Union[Text, bool]]]:
        new_feature_list = []
        for flag_default in config.DEFAULT_FEATURE_FLAGS:
            existing_feature = self._is_contained_in_list(
                flag_default.get("name"), existing_feature_list
            )

            new_feature_list.append(existing_feature or flag_default)
        return new_feature_list

    def features(self) -> List[Dict[Text, Union[Text, bool]]]:
        platform_features = self.session.query(PlatformFeature).all()
        platform_features = [p.as_dict() for p in platform_features]

        return self._merge_with_defaults(platform_features)
